@@ -5,7 +5,7 @@
     Test the autodoc extension.  This tests mainly the Documenters; the auto
     directives are tested in a test source file translated by test_build.
 
-    :copyright: Copyright 2007-2019 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2020 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
@@ -906,7 +906,7 @@ def test_autodoc_module_scope(app):
         '',
         '.. py:attribute:: Class.mdocattr',
         '   :module: target',
-        '   :annotation: = <_io.StringIO object>',
+        '   :value: <_io.StringIO object>',
         '',
         '   should be documented as well - süß',
         '   '
@@ -922,7 +922,7 @@ def test_autodoc_class_scope(app):
         '',
         '.. py:attribute:: Class.mdocattr',
         '   :module: target',
-        '   :annotation: = <_io.StringIO object>',
+        '   :value: <_io.StringIO object>',
         '',
         '   should be documented as well - süß',
         '   '
@@ -942,12 +942,12 @@ def test_class_attributes(app):
         '   ',
         '   .. py:attribute:: AttCls.a1',
         '      :module: target',
-        '      :annotation: = hello world',
+        '      :value: hello world',
         '   ',
         '   ',
         '   .. py:attribute:: AttCls.a2',
         '      :module: target',
-        '      :annotation: = None',
+        '      :value: None',
         '   '
     ]
 
@@ -966,7 +966,7 @@ def test_instance_attributes(app):
         '   ',
         '   .. py:attribute:: InstAttCls.ca1',
         '      :module: target',
-        "      :annotation: = 'a'",
+        "      :value: 'a'",
         '   ',
         '      Doc comment for class attribute InstAttCls.ca1.',
         '      It can have multiple lines.',
@@ -974,28 +974,28 @@ def test_instance_attributes(app):
         '   ',
         '   .. py:attribute:: InstAttCls.ca2',
         '      :module: target',
-        "      :annotation: = 'b'",
+        "      :value: 'b'",
         '   ',
         '      Doc comment for InstAttCls.ca2. One line only.',
         '      ',
         '   ',
         '   .. py:attribute:: InstAttCls.ca3',
         '      :module: target',
-        "      :annotation: = 'c'",
+        "      :value: 'c'",
         '   ',
         '      Docstring for class attribute InstAttCls.ca3.',
         '      ',
         '   ',
         '   .. py:attribute:: InstAttCls.ia1',
         '      :module: target',
-        '      :annotation: = None',
+        '      :value: None',
         '   ',
         '      Doc comment for instance attribute InstAttCls.ia1',
         '      ',
         '   ',
         '   .. py:attribute:: InstAttCls.ia2',
         '      :module: target',
-        '      :annotation: = None',
+        '      :value: None',
         '   ',
         '      Docstring for instance attribute InstAttCls.ia2.',
         '      '
@@ -1014,7 +1014,7 @@ def test_instance_attributes(app):
         '   ',
         '   .. py:attribute:: InstAttCls.ca1',
         '      :module: target',
-        "      :annotation: = 'a'",
+        "      :value: 'a'",
         '   ',
         '      Doc comment for class attribute InstAttCls.ca1.',
         '      It can have multiple lines.',
@@ -1022,7 +1022,7 @@ def test_instance_attributes(app):
         '   ',
         '   .. py:attribute:: InstAttCls.ia1',
         '      :module: target',
-        '      :annotation: = None',
+        '      :value: None',
         '   ',
         '      Doc comment for instance attribute InstAttCls.ia1',
         '      '
@@ -1090,28 +1090,28 @@ def test_enum_class(app):
         '   ',
         '   .. py:attribute:: EnumCls.val1',
         '      :module: target.enum',
-        '      :annotation: = 12',
+        '      :value: 12',
         '   ',
         '      doc for val1',
         '      ',
         '   ',
         '   .. py:attribute:: EnumCls.val2',
         '      :module: target.enum',
-        '      :annotation: = 23',
+        '      :value: 23',
         '   ',
         '      doc for val2',
         '      ',
         '   ',
         '   .. py:attribute:: EnumCls.val3',
         '      :module: target.enum',
-        '      :annotation: = 34',
+        '      :value: 34',
         '   ',
         '      doc for val3',
         '      ',
         '   ',
         '   .. py:attribute:: EnumCls.val4',
         '      :module: target.enum',
-        '      :annotation: = 34',
+        '      :value: 34',
         '   '
     ]
 
@@ -1121,7 +1121,7 @@ def test_enum_class(app):
         '',
         '.. py:attribute:: EnumCls.val1',
         '   :module: target.enum',
-        '   :annotation: = 12',
+        '   :value: 12',
         '',
         '   doc for val1',
         '   '
@@ -1241,19 +1241,25 @@ def test_partialfunction():
         '.. py:module:: target.partialfunction',
         '',
         '',
-        '.. py:function:: func1()',
+        '.. py:function:: func1(a, b, c)',
         '   :module: target.partialfunction',
         '',
         '   docstring of func1',
         '   ',
         '',
-        '.. py:function:: func2()',
+        '.. py:function:: func2(b, c)',
         '   :module: target.partialfunction',
         '',
         '   docstring of func1',
         '   ',
         '',
-        '.. py:function:: func3()',
+        '.. py:function:: func3(c)',
+        '   :module: target.partialfunction',
+        '',
+        '   docstring of func3',
+        '   ',
+        '',
+        '.. py:function:: func4()',
         '   :module: target.partialfunction',
         '',
         '   docstring of func3',
@@ -1261,6 +1267,17 @@ def test_partialfunction():
     ]
 
 
+@pytest.mark.usefixtures('setup_test')
+def test_imported_partialfunction_should_not_shown_without_imported_members():
+    options = {"members": None}
+    actual = do_autodoc(app, 'module', 'target.imported_members', options)
+    assert list(actual) == [
+        '',
+        '.. py:module:: target.imported_members',
+        ''
+    ]
+
+
 @pytest.mark.usefixtures('setup_test')
 def test_bound_method():
     options = {"members": None}
@@ -1318,35 +1335,142 @@ def test_partialmethod(app):
         '   refs: https://docs.python.jp/3/library/functools.html#functools.partialmethod',
         '   ',
         '   ',
-        '   .. py:method:: Cell.set_alive() -> None',
+        '   .. py:method:: Cell.set_alive()',
         '      :module: target.partialmethod',
         '   ',
         '      Make a cell alive.',
         '      ',
         '   ',
-        '   .. py:method:: Cell.set_dead() -> None',
-        '      :module: target.partialmethod',
-        '   ',
-        '      Make a cell dead.',
-        '      ',
-        '   ',
         '   .. py:method:: Cell.set_state(state)',
         '      :module: target.partialmethod',
         '   ',
         '      Update state of cell to *state*.',
         '      ',
     ]
-    if (sys.version_info < (3, 5, 4) or
-            (3, 6, 5) <= sys.version_info < (3, 7) or
-            (3, 7, 0, 'beta', 3) <= sys.version_info):
-        # TODO: this condition should be updated after 3.7-final release.
-        expected = '\n'.join(expected).replace(' -> None', '').split('\n')
 
     options = {"members": None}
     actual = do_autodoc(app, 'class', 'target.partialmethod.Cell', options)
     assert list(actual) == expected
 
 
+@pytest.mark.sphinx('html', testroot='ext-autodoc')
+def test_partialmethod_undoc_members(app):
+    expected = [
+        '',
+        '.. py:class:: Cell',
+        '   :module: target.partialmethod',
+        '',
+        '   An example for partialmethod.',
+        '   ',
+        '   refs: https://docs.python.jp/3/library/functools.html#functools.partialmethod',
+        '   ',
+        '   ',
+        '   .. py:method:: Cell.set_alive()',
+        '      :module: target.partialmethod',
+        '   ',
+        '      Make a cell alive.',
+        '      ',
+        '   ',
+        '   .. py:method:: Cell.set_dead()',
+        '      :module: target.partialmethod',
+        '   ',
+        '   ',
+        '   .. py:method:: Cell.set_state(state)',
+        '      :module: target.partialmethod',
+        '   ',
+        '      Update state of cell to *state*.',
+        '      ',
+    ]
+
+    options = {"members": None,
+               "undoc-members": None}
+    actual = do_autodoc(app, 'class', 'target.partialmethod.Cell', options)
+    assert list(actual) == expected
+
+
+@pytest.mark.skipif(sys.version_info < (3, 6), reason='py36+ is available since python3.6.')
+@pytest.mark.sphinx('html', testroot='ext-autodoc')
+def test_autodoc_typed_instance_variables(app):
+    options = {"members": None,
+               "undoc-members": True}
+    actual = do_autodoc(app, 'module', 'target.typed_vars', options)
+    assert list(actual) == [
+        '',
+        '.. py:module:: target.typed_vars',
+        '',
+        '',
+        '.. py:class:: Class()',
+        '   :module: target.typed_vars',
+        '',
+        '   ',
+        '   .. py:attribute:: Class.attr1',
+        '      :module: target.typed_vars',
+        '      :type: int',
+        '      :value: 0',
+        '   ',
+        '   ',
+        '   .. py:attribute:: Class.attr2',
+        '      :module: target.typed_vars',
+        '      :type: int',
+        '      :value: None',
+        '   ',
+        '   ',
+        '   .. py:attribute:: Class.attr3',
+        '      :module: target.typed_vars',
+        '      :type: int',
+        '      :value: 0',
+        '   ',
+        '   ',
+        '   .. py:attribute:: Class.attr4',
+        '      :module: target.typed_vars',
+        '      :type: int',
+        '      :value: None',
+        '   ',
+        '      attr4',
+        '      ',
+        '   ',
+        '   .. py:attribute:: Class.attr5',
+        '      :module: target.typed_vars',
+        '      :type: int',
+        '      :value: None',
+        '   ',
+        '      attr5',
+        '      ',
+        '   ',
+        '   .. py:attribute:: Class.attr6',
+        '      :module: target.typed_vars',
+        '      :type: int',
+        '      :value: None',
+        '   ',
+        '      attr6',
+        '      ',
+        '',
+        '.. py:data:: attr1',
+        '   :module: target.typed_vars',
+        '   :type: str',
+        "   :value: ''",
+        '',
+        '   attr1',
+        '   ',
+        '',
+        '.. py:data:: attr2',
+        '   :module: target.typed_vars',
+        '   :type: str',
+        '   :value: None',
+        '',
+        '   attr2',
+        '   ',
+        '',
+        '.. py:data:: attr3',
+        '   :module: target.typed_vars',
+        '   :type: str',
+        "   :value: ''",
+        '',
+        '   attr3',
+        '   '
+    ]
+
+
 @pytest.mark.sphinx('html', testroot='pycode-egg')
 def test_autodoc_for_egged_code(app):
     options = {"members": None,
@@ -1359,7 +1483,7 @@ def test_autodoc_for_egged_code(app):
         '',
         '.. py:data:: CONSTANT',
         '   :module: sample',
-        '   :annotation: = 1',
+        '   :value: 1',
         '',
         '   constant on sample.py',
         '   ',
